/* 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version. A copy of the GNU General Public
 License can be found at COPYING.
 
 Many people helped and are helping developing ROCK Linux. Please
 have a look at http://www.rocklinux.org/ and the Documentation/TEAM
 file for details. 
*/

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <string.h>
#include <ctype.h>

const char* VERSION = "0.1";

void show_usage() {
	printf("\nfbstream2rgb version %s\n\n", VERSION);
	printf("Usage: fbstream2rgb [options]\n");
	printf("-i <infile>	select raw data input file	(default: grab.raw)\n");
	printf("-o <outfile>	select rgb data output file	(default: stdout)\n");
	printf("-x <xres>	set x-resolution of input	(default: 640)\n");
	printf("-y <yres>	set y-resolution of input	(default: 480)\n");
	printf("-m <multiplier>	set frame multiplier		(default: 7)\n");
	printf("-h		show this help\n\n");
}
	
int main(int argc, char ** argv)
{
	int in, out;
	char *infile = "grab.raw", *outfile = "-";
	int c, i, m, opt, xres=640, yres=480, multiplier=7;

	while ((opt = getopt (argc, argv, "hi:o:m:y:x:")) != -1)
    switch (opt)
      {
      case 'i':
        infile = optarg;
        break;
      case 'o':
        outfile = optarg;
        break;
      case 'm':
        multiplier = atoi(optarg);
        break;
      case 'x':
        xres = atoi(optarg);
        break;
      case 'y':
        yres = atoi(optarg);
        break;
      case 'h':
        show_usage();
		  return 0;
        break;
      case '?':
        if (isprint(optopt))
          fprintf (stderr, "unknown option `-%c'.\n", optopt);
        else
          fprintf (stderr, "unknown option character `\\x%x'.\n", optopt);
			show_usage();
        return 1;
      default:
        abort();
      }

	unsigned char buf[xres*yres*3];
	unsigned short int fb[xres*yres];

	if(strcmp(infile,"-") == 0) in = 0;
	else {
		in = open(infile, O_RDONLY);
		if( in == -1 ) {
			perror("could not open input file");
			return 1;
		}
	}

	if(strcmp(outfile,"-") == 0) out = 1;
	else {
	  out = open(outfile, O_WRONLY|O_CREAT|O_TRUNC, 0666);
	  if( out == -1 ) {
		  perror("could not open output file");
		  return 1;
	  }
	}

	while ( read(in, fb, sizeof(fb)) == sizeof(fb) )
	{
		for (c=i=0; c<xres*yres; c++) {
			/* Convert gray background to white */
			if ( fb[c] == 0x56b5 ) fb[c] = 0xffff;
			/* Convert raw 15 bit fb data to rgb data */
			buf[i++] = (fb[c] >> 5) & 0xf8;
			buf[i++] = (fb[c]     ) & 0xf8;
			buf[i++] = (fb[c] << 5) & 0xf8;
		}
		for (m=0; m<multiplier; m++)
			write(out, buf, sizeof(buf));
	}
	close(in);
	if(out != 1) {
		close(out);
	}

	return 0;
}

