#!/bin/bash
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version. A copy of the GNU General Public
# License can be found at COPYING.
# 
# Many people helped and are helping developing ROCK Linux. Please
# have a look at http://www.rocklinux.org/ and the Documentation/TEAM
# file for details.


# Use vga=0x310 kernel option for vesafb
# change this to a higher value if video is too fast
MULTIPLIER=7
# how many frames the splash should be visible (25 fps)
SPLASH_FRAMES=70

rm -f rawgrab fbstream2rgb splash.tmp output.m2v *.o grab* core core.*
make CFLAGS="-Wall -O3 " rawgrab fbstream2rgb || exit 1

echo "Recording from framebuffer (run 'killall rawgrab' / press Ctrl+C to stop) .."
./rawgrab || exit 1;

echo; echo -n "Fetching res & framecount ..."
read XRES YRES BPP FRMCOUNT < grab.raw.meta
echo "${XRES}x${YRES}@${BPP}bpp, $FRMCOUNT raw frames"

echo; echo "Creating splash screen ..."
convert -pointsize 30 -draw "gravity North text 0,350 \"$1\"" \
	-pointsize 20 -draw "gravity North text 0,430 \"$2\"" \
	-geometry ${XRES}x${YRES}\! splash.png rgb:splash.tmp

echo; echo "Rendering output.mpeg file ..."
frames=$(( ( $FRMCOUNT * $MULTIPLIER ) + $SPLASH_FRAMES ))

# copy pre-rgbed frames in. add 5 seconds (= 5*25 = 125 images) of splash:
transcode -c 0-$frames -i <(
	for (( i = 0; i <= $SPLASH_FRAMES ; i = i + 1)) ; do
		cat splash.tmp
	done 
	./fbstream2rgb -x $XRES -y $YRES -m $MULTIPLIER
) -x raw,null -g ${XRES}x${YRES} -y mpeg2enc,null -Z 480x576 -F 4 -o output 

echo; echo "Converting to system stream mpeg ..."
mplex -f 4 -S 680 -o output.mpeg output.m2v
rm -f output.m2v grab.raw* splash.tmp

echo DONE.

